%% ========================================================================
% Test of LPempc: Energy system case study
% Written by: Leo Emil Sokoler, leso@imm.dtu.dk
% 02/05/2013, DTU Informatics
% =========================================================================
clear; clc; close all;
rng('default');

%% ========================================================================
% SYSTEM PARAMTERS
% =========================================================================
m = 5;                      % nr. of units
Tau = 30 + rand(m,1)*50;    % time constants

%% ========================================================================
% SYSTEM SETUP
% =========================================================================
Ts = 5;
[A,B,E,Cy,Cz] = ESsystem(Tau,Ts);
[nx,nu] = size(B);
C = Cy;
ny = size(C,1);
nd = size(E,2);
x0 = sum(C,1)'.*rand(nx,1);
u_m1 = randn(nu,1);

%% ========================================================================
% CONTROLLER SETUP
%  ========================================================================
% paramters
N = 120;                        % prediction horizon
p = 1./Tau;                     % input costs
q = 1e4*ones(ny,1);             % soft constraints penalty
u_min = zeros(nu,1);            % min input
u_max = 200*rand(m,1) + 20;     % max input
Du_min = -20*rand(m,1) - 20;    % min input rate
Du_max = 20*rand(m,1) + 20;     % max input rate
y_min = zeros(ny,N);            % min output
y_max = zeros(ny,N)+300;        % max output

%% ========================================================================
% SCNEARIO SETUP
% =========================================================================
T = 0:Ts:Ts*N;                 % time vector
P = zeros(N,1);
M = floor(N/4);
P(1:M) = linspace(0,20,M);
P(M:end) = 20*ones(length(P(M:end)),1);
y_min(end,:) = max(P - 5,0);
y_max(end,:) = P + 5;
d = zeros(nd,N);

%% ========================================================================
% SOLVE OPTIMIZATION PROBLEM
% =========================================================================
[u,x,y,rho,info,output] = LPempc(A,B,E,C,d,x0,u_m1,u_min, ...
    u_max,Du_min,Du_max,y_min,y_max,p,q,N);


%% ========================================================================
% PLOT TOTAL POWER PRODUCTION
% =========================================================================
subplot(2,1,1);
plot(T(2:end),Cz*x); grid on;  hold on;
plot(T(2:end),y_min(end,:), 'r--'); 
plot(T(2:end),y_max(end,:), 'r--');
ylabel('C_z');
subplot(2,1,2);
stairs(T(2:end),u'); grid on;
ylabel('U');
xlabel('time [sec]'); 

