%% ========================================================================
% Test of LPempc: Mass-spring system case study
% Written by: Leo Emil Sokoler, leso@imm.dtu.dk
% 02/05/2013, DTU Informatics
% =========================================================================
clear; clc; close all;
rng('default');

%% ========================================================================
% SYSTEM PARAMTERS
% =========================================================================
nm = 4;                      % nr. of masses
na = 2;                      % nr. of actuators

%% ========================================================================
% SYSTEM SETUP
% =========================================================================
Ts = 0.2;
[A,B,E,Cy,Cz] = MSsystem(nm, na, Ts);
[nx,nu] = size(B);
C = Cy;
ny = size(C,1);
nd = size(E,2);
x0 = [randn(nm,1); zeros(nm,1)];
u_m1 = zeros(nu,1);

%% ========================================================================
% CONTROLLER SETUP
%  ========================================================================
% paramters
N = 120;                            % prediction horizon
p = flipud([1./(1:na) 1./(1:na)]'); % input costs
q = 1e5*ones(ny,1);                 % soft constraints penalty
u_min = zeros(nu,1);                % min input
u_max = 0.5*ones(nu,1);             % max input
Du_min = -u_max/4;                  % min input rate
Du_max = u_max/4;                   % max input rate
y_min = -0.05*ones(ny,1);           % min output
y_max = 0.05*ones(ny,1);            % max output
d = zeros(nd,1);                    % disturbance

%% ========================================================================
% SOLVE OPTIMIZATION PROBLEM
% =========================================================================
[u,x,y,rho,info,output] = LPempc(A,B,E,C,d,x0,u_m1,u_min, ...
    u_max,Du_min,Du_max,y_min,y_max,p,q,N);


%% ========================================================================
% PLOT SOLUTION
% =========================================================================
T = 0:Ts:Ts*N;                 % time vector
subplot(2,1,1);
plot(T(2:end),y'); grid on;  hold on;
ylabel('C_z');
subplot(2,1,2);
stairs(T(2:end),u'); grid on;  hold on;
xlabel('time, [sec]');
ylabel('U');



